@@ -6,13 +6,14 @@ def scramble(email, exam, *, keep_data=False):
 
     def scramble_group(group, substitutions, config, depth):
         group_substitutions = select(group["substitutions"])
+        group_substitutions.update(select_no_replace(group.get("substitutions_match", [])))
         substitute(
             group,
             [*substitutions, group_substitutions],
             ["name", "html", "tex", "text"],
         )
         if depth in config["scramble_groups"] or group.get("scramble"):
-            random.shuffle(get_elements(group))
+            scramble_keep_fixed(get_elements(group))
         if group.get("pick_some"):
             get_elements(group)[:] = random.sample(
                 get_elements(group), group["pick_some"]
@@ -29,11 +30,12 @@ def scramble(email, exam, *, keep_data=False):
 
     def scramble_question(question, substitutions, config):
         question_substitutions = select(question["substitutions"])
+        question_substitutions.update(select_no_replace(question.get("substitutions_match", [])))
         substitute(
             question, [question_substitutions, *substitutions], ["html", "tex", "text"]
         )
         if "scramble_options" in config and isinstance(question["options"], list):
-            scramble_options(question["options"])
+            scramble_keep_fixed(question["options"])
             for option in question["options"]:
                 substitute(
                     option,
@@ -55,11 +57,12 @@ def scramble(email, exam, *, keep_data=False):
             target.pop("substitutions", None)
 
     global_substitutions = select(exam["substitutions"])
+    global_substitutions.update(select_no_replace(exam.get("substitutions_match", [])))
     exam["config"]["scramble_groups"] = exam["config"].get(
         "scramble_groups", [-1]
     ) or range(100)
     if 0 in exam["config"]["scramble_groups"]:
-        random.shuffle(exam["groups"])
+        scramble_keep_fixed(exam["groups"])
     for group in exam["groups"]:
         scramble_group(group, [global_substitutions], exam["config"], 1)
     exam.pop("config", None)
@@ -67,16 +70,16 @@ def scramble(email, exam, *, keep_data=False):
     return exam
 
 
-def scramble_options(options):
-    movable_option_pos = []
-    movable_option_values = []
-    for i, option in enumerate(options):
-        if not option.get("fixed"):
-            movable_option_pos.append(i)
-            movable_option_values.append(option)
-    random.shuffle(movable_option_values)
-    for i, option in zip(movable_option_pos, movable_option_values):
-        options[i] = option
+def scramble_keep_fixed(objects):
+    movable_object_pos = []
+    movable_object_values = []
+    for i, object in enumerate(objects):
+        if not object.get("fixed"):
+            movable_object_pos.append(i)
+            movable_object_values.append(object)
+    random.shuffle(movable_object_values)
+    for i, object in zip(movable_object_pos, movable_object_values):
+        objects[i] = object
 
 
 def get_elements(group):
@@ -85,6 +88,20 @@ def get_elements(group):
 
 def select(substitutions):
     out = {}
+    # DEFINE
     for k, v in sorted(substitutions.items()):
         out[k] = random.choice(v)
     return out
+
+def select_no_replace(substitutions_match):
+    out = {}
+    # DEFINE MATCH
+    for item in substitutions_match:
+        k = item["directives"]
+        v = item["replacements"]
+        values = v.copy()
+        for choice in k:
+            c = random.choice(values)
+            values.remove(c)
+            out[choice] = c
+    return out
